// CS 240
// Jace Miller
// particle.h

#ifndef __PARTICLE_H__
#define __PARTICLE_H__

#include <stdlib.h>

/******************************************************************************/

// A simple 3D vector.

class Vector
{
public:
	Vector() { mX = 0; mY = 0; mZ = 0; };
	Vector(float x, float y, float z) { mX = x; mY = y; mZ = z; };
	void set(float x, float y, float z) { mX = x; mY = y; mZ = z; };
	void add(const Vector& delta) { mX += delta.mX; mY += delta.mY; mZ += delta.mZ; };
	float getX() { return mX; };
	float getY() { return mY; };
	float getZ() { return mZ; };

public:
	float mX;
	float mY;
	float mZ;
};

/******************************************************************************/

// A Particle is a point with physical attributes describing motion and color.

class Particle
{
public:
	void setPosition(float x, float y, float z) { mPosition.set(x, y, z); };
	void setVelocity(float x, float y, float z) { mVelocity.set(x, y, z); };
	void setAcceleration(float x, float y, float z) { mAcceleration.set(x, y, z); };
	void setColor(float r, float g, float b) { mColor.set(r, g, b); };
	void respawn();
	void update();

protected:
	Vector mPosition;
	Vector mVelocity;
	Vector mAcceleration;
	Vector mColor;
};

/******************************************************************************/

// Drawable is the idea I want captured as an interface.

class Drawable
{
public:
	virtual void draw() = 0;
	virtual void update() = 0;
};


/******************************************************************************/

// Since I need to make an interface specific to every object that is drawable,
// I use a template to save coding effort.

template <class T> class DrawableInterface : public Drawable
{
private:
	T* d;

public:
	DrawableInterface(T* pT) { d=pT; };
	void draw() { d->draw(); };
	void update() { d->update(); };
};

/******************************************************************************/

// A Point is a particle that can be drawn.

class Point : public Particle
{
public:
	Point() : drawable(this) { };
	void draw();
	Drawable * getDrawable() { return &drawable; };

private:
	DrawableInterface<Point> drawable;
};

/******************************************************************************/

// A Line is a particle that can be drawn (differently than a point).

class Line : public Particle
{
public:
	Line() : drawable(this) { };
	void draw();
	Drawable * getDrawable() { return &drawable; };
private:
	DrawableInterface<Line> drawable;
};


#endif
